@@ -109,6 +109,20 @@ class OpTest(TestBase):
         op.add_column("t1", Column("c1", Integer, nullable=False))
         context.assert_("ALTER TABLE t1 ADD COLUMN c1 INTEGER NOT NULL")
 
+    def test_add_column_exists_directives(self):
+        context = op_fixture()
+        op.add_column(
+            "t1", Column("c1", Integer, nullable=False), if_not_exists=True
+        )
+        context.assert_(
+            "ALTER TABLE t1 ADD COLUMN IF NOT EXISTS c1 INTEGER NOT NULL"
+        )
+
+    def test_drop_column_exists_directives(self):
+        context = op_fixture()
+        op.drop_column("t1", "c1", if_exists=True)
+        context.assert_("ALTER TABLE t1 DROP COLUMN IF EXISTS c1")
+
     def test_add_column_already_attached(self):
         context = op_fixture()
         c1 = Column("c1", Integer, nullable=False)
@@ -821,6 +835,19 @@ class OpTest(TestBase):
         op.drop_constraint("foo_bar_bat", "t1", schema="foo")
         context.assert_("ALTER TABLE foo.t1 DROP CONSTRAINT foo_bar_bat")
 
+    def test_drop_constraint_if_exists(self):
+        context = op_fixture()
+        if sqla_compat.sqla_2:
+            op.drop_constraint("foo_bar_bat", "t1", if_exists=True)
+            context.assert_(
+                "ALTER TABLE t1 DROP CONSTRAINT IF EXISTS foo_bar_bat"
+            )
+        else:
+            with expect_raises_message(
+                NotImplementedError, "SQLAlchemy 2.0 required"
+            ):
+                op.drop_constraint("foo_bar_bat", "t1", if_exists=True)
+
     def test_create_index(self):
         context = op_fixture()
         op.create_index("ik_test", "t1", ["foo", "bar"])
